# generated by datamodel-codegen:
#   filename:  computebackendservice_v1beta1.json
#   timestamp: 2024-03-15T04:51:32+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field


class BackendItem(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    balancingMode: Optional[str] = Field(
        None,
        description='Specifies the balancing mode for this backend.\n\nFor global HTTP(S) or TCP/SSL load balancing, the default is\nUTILIZATION. Valid values are UTILIZATION, RATE (for HTTP(S))\nand CONNECTION (for TCP/SSL).\n\nSee the [Backend Services Overview](https://cloud.google.com/load-balancing/docs/backend-service#balancing-mode)\nfor an explanation of load balancing modes. Default value: "UTILIZATION" Possible values: ["UTILIZATION", "RATE", "CONNECTION"].',
    )
    capacityScaler: Optional[float] = Field(
        None,
        description="A multiplier applied to the group's maximum servicing capacity\n(based on UTILIZATION, RATE or CONNECTION).\n\nDefault value is 1, which means the group will serve up to 100%\nof its configured capacity (depending on balancingMode). A\nsetting of 0 means the group is completely drained, offering\n0% of its available Capacity. Valid range is [0.0,1.0].",
    )
    description: Optional[str] = Field(
        None,
        description='An optional description of this resource.\nProvide this property when you create the resource.',
    )
    failover: Optional[bool] = Field(
        None,
        description='This field designates whether this is a failover backend. More\nthan one failover backend can be configured for a given RegionBackendService.',
    )
    group: Union[Group, Group1] = Field(
        ...,
        description="Reference to a ComputeInstanceGroup or ComputeNetworkEndpointGroup\nresource. In case of instance group this defines the list of\ninstances that serve traffic. Member virtual machine instances from\neach instance group must live in the same zone as the instance\ngroup itself. No two backends in a backend service are allowed to\nuse same Instance Group resource.\n\nFor Network Endpoint Groups this defines list of endpoints. All\nendpoints of Network Endpoint Group must be hosted on instances\nlocated in the same zone as the Network Endpoint Group.\n\nBackend services cannot mix Instance Group and Network Endpoint\nGroup backends.\n\nWhen the 'load_balancing_scheme' is INTERNAL, only instance groups\nare supported.",
    )
    maxConnections: Optional[int] = Field(
        None,
        description='The max number of simultaneous connections for the group. Can\nbe used with either CONNECTION or UTILIZATION balancing modes.\n\nFor CONNECTION mode, either maxConnections or one\nof maxConnectionsPerInstance or maxConnectionsPerEndpoint,\nas appropriate for group type, must be set.',
    )
    maxConnectionsPerEndpoint: Optional[int] = Field(
        None,
        description='The max number of simultaneous connections that a single backend\nnetwork endpoint can handle. This is used to calculate the\ncapacity of the group. Can be used in either CONNECTION or\nUTILIZATION balancing modes.\n\nFor CONNECTION mode, either\nmaxConnections or maxConnectionsPerEndpoint must be set.',
    )
    maxConnectionsPerInstance: Optional[int] = Field(
        None,
        description='The max number of simultaneous connections that a single\nbackend instance can handle. This is used to calculate the\ncapacity of the group. Can be used in either CONNECTION or\nUTILIZATION balancing modes.\n\nFor CONNECTION mode, either maxConnections or\nmaxConnectionsPerInstance must be set.',
    )
    maxRate: Optional[int] = Field(
        None,
        description='The max requests per second (RPS) of the group.\n\nCan be used with either RATE or UTILIZATION balancing modes,\nbut required if RATE mode. For RATE mode, either maxRate or one\nof maxRatePerInstance or maxRatePerEndpoint, as appropriate for\ngroup type, must be set.',
    )
    maxRatePerEndpoint: Optional[float] = Field(
        None,
        description='The max requests per second (RPS) that a single backend network\nendpoint can handle. This is used to calculate the capacity of\nthe group. Can be used in either balancing mode. For RATE mode,\neither maxRate or maxRatePerEndpoint must be set.',
    )
    maxRatePerInstance: Optional[float] = Field(
        None,
        description='The max requests per second (RPS) that a single backend\ninstance can handle. This is used to calculate the capacity of\nthe group. Can be used in either balancing mode. For RATE mode,\neither maxRate or maxRatePerInstance must be set.',
    )
    maxUtilization: Optional[float] = Field(
        None,
        description='Used when balancingMode is UTILIZATION. This ratio defines the\nCPU utilization target for the group. Valid range is [0.0, 1.0].',
    )


class BaseEjectionTime(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    nanos: Optional[int] = Field(
        None,
        description="Span of time that's a fraction of a second at nanosecond resolution. Durations\nless than one second are represented with a 0 'seconds' field and a positive\n'nanos' field. Must be from 0 to 999,999,999 inclusive.",
    )
    seconds: int = Field(
        ...,
        description='Span of time at a resolution of a second. Must be from 0 to 315,576,000,000\ninclusive.',
    )


class BypassCacheOnRequestHeader(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    headerName: str = Field(
        ...,
        description='The header field name to match on when bypassing cache. Values are case-insensitive.',
    )


class CacheKeyPolicy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    includeHost: Optional[bool] = Field(
        None,
        description='If true requests to different hosts will be cached separately.',
    )
    includeHttpHeaders: Optional[List[str]] = Field(
        None,
        description='Allows HTTP request headers (by name) to be used in the\ncache key.',
    )
    includeNamedCookies: Optional[List[str]] = Field(
        None, description='Names of cookies to include in cache keys.'
    )
    includeProtocol: Optional[bool] = Field(
        None, description='If true, http and https requests will be cached separately.'
    )
    includeQueryString: Optional[bool] = Field(
        None,
        description='If true, include query string parameters in the cache key\naccording to query_string_whitelist and\nquery_string_blacklist. If neither is set, the entire query\nstring will be included.\n\nIf false, the query string will be excluded from the cache\nkey entirely.',
    )
    queryStringBlacklist: Optional[List[str]] = Field(
        None,
        description="Names of query string parameters to exclude in cache keys.\n\nAll other parameters will be included. Either specify\nquery_string_whitelist or query_string_blacklist, not both.\n'&' and '=' will be percent encoded and not treated as\ndelimiters.",
    )
    queryStringWhitelist: Optional[List[str]] = Field(
        None,
        description="Names of query string parameters to include in cache keys.\n\nAll other parameters will be excluded. Either specify\nquery_string_whitelist or query_string_blacklist, not both.\n'&' and '=' will be percent encoded and not treated as\ndelimiters.",
    )


class CdnPolicy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    bypassCacheOnRequestHeaders: Optional[List[BypassCacheOnRequestHeader]] = Field(
        None,
        description='Bypass the cache when the specified request headers are matched - e.g. Pragma or Authorization headers. Up to 5 headers can be specified.\nThe cache is bypassed for all cdnPolicy.cacheMode settings.',
    )
    cacheKeyPolicy: Optional[CacheKeyPolicy] = Field(
        None, description='The CacheKeyPolicy for this CdnPolicy.'
    )
    cacheMode: Optional[str] = Field(
        None,
        description='Specifies the cache setting for all responses from this backend.\nThe possible values are: USE_ORIGIN_HEADERS, FORCE_CACHE_ALL and CACHE_ALL_STATIC Possible values: ["USE_ORIGIN_HEADERS", "FORCE_CACHE_ALL", "CACHE_ALL_STATIC"].',
    )
    clientTtl: Optional[int] = Field(
        None,
        description='Specifies the maximum allowed TTL for cached content served by this origin.',
    )
    defaultTtl: Optional[int] = Field(
        None,
        description='Specifies the default TTL for cached content served by this origin for responses\nthat do not have an existing valid TTL (max-age or s-max-age).',
    )
    maxTtl: Optional[int] = Field(
        None,
        description='Specifies the maximum allowed TTL for cached content served by this origin.',
    )
    negativeCaching: Optional[bool] = Field(
        None,
        description='Negative caching allows per-status code TTLs to be set, in order to apply fine-grained caching for common errors or redirects.',
    )
    negativeCachingPolicy: Optional[List[NegativeCachingPolicyItem]] = Field(
        None,
        description="Sets a cache TTL for the specified HTTP status code. negativeCaching must be enabled to configure negativeCachingPolicy.\nOmitting the policy and leaving negativeCaching enabled will use Cloud CDN's default cache TTLs.",
    )
    serveWhileStale: Optional[int] = Field(
        None,
        description='Serve existing content from the cache (if available) when revalidating content with the origin, or when an error is encountered when refreshing the cache.',
    )
    signedUrlCacheMaxAgeSec: Optional[int] = Field(
        None,
        description='Maximum number of seconds the response to a signed URL request\nwill be considered fresh, defaults to 1hr (3600s). After this\ntime period, the response will be revalidated before\nbeing served.\n\nWhen serving responses to signed URL requests, Cloud CDN will\ninternally behave as though all responses from this backend had a\n"Cache-Control: public, max-age=[TTL]" header, regardless of any\nexisting Cache-Control header. The actual headers served in\nresponses will not be altered.',
    )


class CircuitBreakers(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    connectTimeout: Optional[ConnectTimeout] = Field(
        None, description='The timeout for new network connections to hosts.'
    )
    maxConnections: Optional[int] = Field(
        None,
        description='The maximum number of connections to the backend cluster.\nDefaults to 1024.',
    )
    maxPendingRequests: Optional[int] = Field(
        None,
        description='The maximum number of pending requests to the backend cluster.\nDefaults to 1024.',
    )
    maxRequests: Optional[int] = Field(
        None,
        description='The maximum number of parallel requests to the backend cluster.\nDefaults to 1024.',
    )
    maxRequestsPerConnection: Optional[int] = Field(
        None,
        description='Maximum requests for a single backend connection. This parameter\nis respected by both the HTTP/1.1 and HTTP/2 implementations. If\nnot specified, there is no limit. Setting this parameter to 1\nwill effectively disable keep alive.',
    )
    maxRetries: Optional[int] = Field(
        None,
        description='The maximum number of parallel retries to the backend cluster.\nDefaults to 3.',
    )


class ClientTLSPolicyRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `name` field of a `NetworkSecurityClientTLSPolicy` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class ClientTLSPolicyRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `name` field of a `NetworkSecurityClientTLSPolicy` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Condition(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    lastTransitionTime: Optional[str] = Field(
        None,
        description='Last time the condition transitioned from one status to another.',
    )
    message: Optional[str] = Field(
        None,
        description='Human-readable message indicating details about last transition.',
    )
    reason: Optional[str] = Field(
        None,
        description="Unique, one-word, CamelCase reason for the condition's last transition.",
    )
    status: Optional[str] = Field(
        None,
        description='Status is the status of the condition. Can be True, False, Unknown.',
    )
    type: Optional[str] = Field(None, description='Type is the type of the condition.')


class ConnectTimeout(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    nanos: Optional[int] = Field(
        None,
        description="Span of time that's a fraction of a second at nanosecond\nresolution. Durations less than one second are represented\nwith a 0 seconds field and a positive nanos field. Must\nbe from 0 to 999,999,999 inclusive.",
    )
    seconds: int = Field(
        ...,
        description='Span of time at a resolution of a second.\nMust be from 0 to 315,576,000,000 inclusive.',
    )


class ConnectionTrackingPolicy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    connectionPersistenceOnUnhealthyBackends: Optional[str] = Field(
        None,
        description='Specifies connection persistence when backends are unhealthy.\n\nIf set to \'DEFAULT_FOR_PROTOCOL\', the existing connections persist on\nunhealthy backends only for connection-oriented protocols (TCP and SCTP)\nand only if the Tracking Mode is PER_CONNECTION (default tracking mode)\nor the Session Affinity is configured for 5-tuple. They do not persist\nfor UDP.\n\nIf set to \'NEVER_PERSIST\', after a backend becomes unhealthy, the existing\nconnections on the unhealthy backend are never persisted on the unhealthy\nbackend. They are always diverted to newly selected healthy backends\n(unless all backends are unhealthy).\n\nIf set to \'ALWAYS_PERSIST\', existing connections always persist on\nunhealthy backends regardless of protocol and session affinity. It is\ngenerally not recommended to use this mode overriding the default. Default value: "DEFAULT_FOR_PROTOCOL" Possible values: ["DEFAULT_FOR_PROTOCOL", "NEVER_PERSIST", "ALWAYS_PERSIST"].',
    )
    enableStrongAffinity: Optional[bool] = Field(
        None,
        description='Enable Strong Session Affinity for Network Load Balancing. This option is not available publicly.',
    )
    idleTimeoutSec: Optional[int] = Field(
        None,
        description='Specifies how long to keep a Connection Tracking entry while there is\nno matching traffic (in seconds).\n\nFor L4 ILB the minimum(default) is 10 minutes and maximum is 16 hours.\n\nFor NLB the minimum(default) is 60 seconds and the maximum is 16 hours.',
    )
    trackingMode: Optional[str] = Field(
        None,
        description='Specifies the key used for connection tracking. There are two options:\n\'PER_CONNECTION\': The Connection Tracking is performed as per the\nConnection Key (default Hash Method) for the specific protocol.\n\n\'PER_SESSION\': The Connection Tracking is performed as per the\nconfigured Session Affinity. It matches the configured Session Affinity. Default value: "PER_CONNECTION" Possible values: ["PER_CONNECTION", "PER_SESSION"].',
    )


class ConsistentHash(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    httpCookie: Optional[HttpCookie] = Field(
        None,
        description='Hash is based on HTTP Cookie. This field describes a HTTP cookie\nthat will be used as the hash key for the consistent hash load\nbalancer. If the cookie is not present, it will be generated.\nThis field is applicable if the sessionAffinity is set to HTTP_COOKIE.',
    )
    httpHeaderName: Optional[str] = Field(
        None,
        description='The hash based on the value of the specified header field.\nThis field is applicable if the sessionAffinity is set to HEADER_FIELD.',
    )
    minimumRingSize: Optional[int] = Field(
        None,
        description='The minimum number of virtual nodes to use for the hash ring.\nLarger ring sizes result in more granular load\ndistributions. If the number of hosts in the load balancing pool\nis larger than the ring size, each host will be assigned a single\nvirtual node.\nDefaults to 1024.',
    )


class CustomPolicy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    data: Optional[str] = Field(
        None,
        description='An optional, arbitrary JSON object with configuration data, understood\nby a locally installed custom policy implementation.',
    )
    name: str = Field(
        ...,
        description='Identifies the custom policy.\n\nThe value should match the type the custom implementation is registered\nwith on the gRPC clients. It should follow protocol buffer\nmessage naming conventions and include the full path (e.g.\nmyorg.CustomLbPolicy). The maximum length is 256 characters.\n\nNote that specifying the same custom policy more than once for a\nbackend is not a valid configuration and will be rejected.',
    )


class EdgeSecurityPolicyRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeSecurityPolicy` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class EdgeSecurityPolicyRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeSecurityPolicy` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class FailoverPolicy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    disableConnectionDrainOnFailover: Optional[bool] = Field(
        None,
        description='On failover or failback, this field indicates whether connection drain\nwill be honored. Setting this to true has the following effect: connections\nto the old active pool are not drained. Connections to the new active pool\nuse the timeout of 10 min (currently fixed). Setting to false has the\nfollowing effect: both old and new connections will have a drain timeout\nof 10 min.\nThis can be set to true only if the protocol is TCP.\nThe default is false.',
    )
    dropTrafficIfUnhealthy: Optional[bool] = Field(
        None,
        description='This option is used only when no healthy VMs are detected in the primary\nand backup instance groups. When set to true, traffic is dropped. When\nset to false, new connections are sent across all VMs in the primary group.\nThe default is false.',
    )
    failoverRatio: Optional[float] = Field(
        None,
        description='The value of the field must be in [0, 1]. If the ratio of the healthy\nVMs in the primary backend is at or below this number, traffic arriving\nat the load-balanced IP will be directed to the failover backend.\nIn case where \'failoverRatio\' is not set or all the VMs in the backup\nbackend are unhealthy, the traffic will be directed back to the primary\nbackend in the "force" mode, where traffic will be spread to the healthy\nVMs with the best effort, or to all VMs when no VM is healthy.\nThis field is only used with l4 load balancing.',
    )


class Group(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    instanceGroupRef: Union[InstanceGroupRef, InstanceGroupRef1]
    networkEndpointGroupRef: Optional[
        Union[NetworkEndpointGroupRef, NetworkEndpointGroupRef1]
    ] = None


class Group1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    instanceGroupRef: Optional[Union[InstanceGroupRef2, InstanceGroupRef3]] = None
    networkEndpointGroupRef: Union[NetworkEndpointGroupRef2, NetworkEndpointGroupRef3]


class HealthCheckRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeHealthCheck` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class HealthCheckRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeHealthCheck` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class HealthCheckRef2(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeHealthCheck` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class HealthCheckRef3(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeHealthCheck` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class HealthChecks(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    healthCheckRef: Union[HealthCheckRef, HealthCheckRef1]
    httpHealthCheckRef: Optional[Union[HttpHealthCheckRef, HttpHealthCheckRef1]] = None


class HealthChecks1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    healthCheckRef: Optional[Union[HealthCheckRef2, HealthCheckRef3]] = None
    httpHealthCheckRef: Union[HttpHealthCheckRef2, HttpHealthCheckRef3]


class HttpCookie(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    name: Optional[str] = Field(None, description='Name of the cookie.')
    path: Optional[str] = Field(None, description='Path to set for the cookie.')
    ttl: Optional[Ttl] = Field(None, description='Lifetime of the cookie.')


class HttpHealthCheckRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeHTTPHealthCheck` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class HttpHealthCheckRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeHTTPHealthCheck` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class HttpHealthCheckRef2(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeHTTPHealthCheck` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class HttpHealthCheckRef3(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeHTTPHealthCheck` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Iap(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    oauth2ClientId: str = Field(
        ...,
        description='DEPRECATED. Although this field is still available, there is limited support. We recommend that you use `spec.iap.oauth2ClientIdRef` instead.',
    )
    oauth2ClientIdRef: Optional[Union[Oauth2ClientIdRef, Oauth2ClientIdRef1]] = Field(
        None, description='OAuth2 Client ID for IAP.'
    )
    oauth2ClientSecret: Optional[Union[Oauth2ClientSecret, Oauth2ClientSecret1]] = (
        Field(None, description='OAuth2 Client Secret for IAP.')
    )
    oauth2ClientSecretSha256: Optional[str] = Field(
        None, description='OAuth2 Client Secret SHA-256 for IAP.'
    )


class Iap1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    oauth2ClientId: Optional[str] = Field(
        None,
        description='DEPRECATED. Although this field is still available, there is limited support. We recommend that you use `spec.iap.oauth2ClientIdRef` instead.',
    )
    oauth2ClientIdRef: Union[Oauth2ClientIdRef2, Oauth2ClientIdRef3] = Field(
        ..., description='OAuth2 Client ID for IAP.'
    )
    oauth2ClientSecret: Optional[Union[Oauth2ClientSecret2, Oauth2ClientSecret3]] = (
        Field(None, description='OAuth2 Client Secret for IAP.')
    )
    oauth2ClientSecretSha256: Optional[str] = Field(
        None, description='OAuth2 Client Secret SHA-256 for IAP.'
    )


class InstanceGroupRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeInstanceGroup` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class InstanceGroupRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeInstanceGroup` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class InstanceGroupRef2(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeInstanceGroup` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class InstanceGroupRef3(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeInstanceGroup` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Interval(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    nanos: Optional[int] = Field(
        None,
        description="Span of time that's a fraction of a second at nanosecond resolution. Durations\nless than one second are represented with a 0 'seconds' field and a positive\n'nanos' field. Must be from 0 to 999,999,999 inclusive.",
    )
    seconds: int = Field(
        ...,
        description='Span of time at a resolution of a second. Must be from 0 to 315,576,000,000\ninclusive.',
    )


class LocalityLbPolicy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    customPolicy: Optional[CustomPolicy] = Field(
        None,
        description='The configuration for a custom policy implemented by the user and\ndeployed with the client.',
    )
    policy: Optional[Policy] = Field(
        None, description='The configuration for a built-in load balancing policy.'
    )


class LogConfig(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    enable: Optional[bool] = Field(
        None,
        description='Whether to enable logging for the load balancer traffic served by this backend service.',
    )
    sampleRate: Optional[float] = Field(
        None,
        description='This field can only be specified if logging is enabled for this backend service. The value of\nthe field must be in [0, 1]. This configures the sampling rate of requests to the load balancer\nwhere 1.0 means all logged requests are reported and 0.0 means no logged requests are reported.\nThe default value is 1.0.',
    )


class Model(BaseModel):
    apiVersion: Optional[str] = Field(
        None,
        description='apiVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources',
    )
    kind: Optional[str] = Field(
        None,
        description='kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds',
    )
    metadata: Optional[Dict[str, Any]] = None
    spec: Spec
    status: Optional[Status] = None


class NegativeCachingPolicyItem(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    code: Optional[int] = Field(
        None,
        description='The HTTP status code to define a TTL against. Only HTTP status codes 300, 301, 308, 404, 405, 410, 421, 451 and 501\ncan be specified as values, and you cannot specify a status code more than once.',
    )
    ttl: Optional[int] = Field(
        None,
        description='The TTL (in seconds) for which to cache responses with the corresponding status code. The maximum allowed value is 1800s\n(30 minutes), noting that infrequently accessed objects may be evicted from the cache before the defined TTL.',
    )


class NetworkEndpointGroupRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeNetworkEndpointGroup` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class NetworkEndpointGroupRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeNetworkEndpointGroup` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class NetworkEndpointGroupRef2(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeNetworkEndpointGroup` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class NetworkEndpointGroupRef3(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeNetworkEndpointGroup` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class NetworkRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeNetwork` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class NetworkRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeNetwork` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Oauth2ClientIdRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `name` field of an `IAPIdentityAwareProxyClient` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Oauth2ClientIdRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `name` field of an `IAPIdentityAwareProxyClient` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Oauth2ClientIdRef2(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `name` field of an `IAPIdentityAwareProxyClient` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Oauth2ClientIdRef3(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `name` field of an `IAPIdentityAwareProxyClient` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Oauth2ClientSecret(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    value: str = Field(
        ...,
        description="Value of the field. Cannot be used if 'valueFrom' is specified.",
    )
    valueFrom: Optional[ValueFrom] = Field(
        None,
        description="Source for the field's value. Cannot be used if 'value' is specified.",
    )


class Oauth2ClientSecret1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    value: Optional[str] = Field(
        None,
        description="Value of the field. Cannot be used if 'valueFrom' is specified.",
    )
    valueFrom: ValueFrom1 = Field(
        ...,
        description="Source for the field's value. Cannot be used if 'value' is specified.",
    )


class Oauth2ClientSecret2(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    value: str = Field(
        ...,
        description="Value of the field. Cannot be used if 'valueFrom' is specified.",
    )
    valueFrom: Optional[ValueFrom2] = Field(
        None,
        description="Source for the field's value. Cannot be used if 'value' is specified.",
    )


class Oauth2ClientSecret3(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    value: Optional[str] = Field(
        None,
        description="Value of the field. Cannot be used if 'valueFrom' is specified.",
    )
    valueFrom: ValueFrom3 = Field(
        ...,
        description="Source for the field's value. Cannot be used if 'value' is specified.",
    )


class OutlierDetection(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    baseEjectionTime: Optional[BaseEjectionTime] = Field(
        None,
        description='The base time that a host is ejected for. The real time is equal to the base\ntime multiplied by the number of times the host has been ejected. Defaults to\n30000ms or 30s.',
    )
    consecutiveErrors: Optional[int] = Field(
        None,
        description='Number of errors before a host is ejected from the connection pool. When the\nbackend host is accessed over HTTP, a 5xx return code qualifies as an error.\nDefaults to 5.',
    )
    consecutiveGatewayFailure: Optional[int] = Field(
        None,
        description='The number of consecutive gateway failures (502, 503, 504 status or connection\nerrors that are mapped to one of those status codes) before a consecutive\ngateway failure ejection occurs. Defaults to 5.',
    )
    enforcingConsecutiveErrors: Optional[int] = Field(
        None,
        description='The percentage chance that a host will be actually ejected when an outlier\nstatus is detected through consecutive 5xx. This setting can be used to disable\nejection or to ramp it up slowly. Defaults to 100.',
    )
    enforcingConsecutiveGatewayFailure: Optional[int] = Field(
        None,
        description='The percentage chance that a host will be actually ejected when an outlier\nstatus is detected through consecutive gateway failures. This setting can be\nused to disable ejection or to ramp it up slowly. Defaults to 0.',
    )
    enforcingSuccessRate: Optional[int] = Field(
        None,
        description='The percentage chance that a host will be actually ejected when an outlier\nstatus is detected through success rate statistics. This setting can be used to\ndisable ejection or to ramp it up slowly. Defaults to 100.',
    )
    interval: Optional[Interval] = Field(
        None,
        description='Time interval between ejection sweep analysis. This can result in both new\nejections as well as hosts being returned to service. Defaults to 10 seconds.',
    )
    maxEjectionPercent: Optional[int] = Field(
        None,
        description='Maximum percentage of hosts in the load balancing pool for the backend service\nthat can be ejected. Defaults to 10%.',
    )
    successRateMinimumHosts: Optional[int] = Field(
        None,
        description='The number of hosts in a cluster that must have enough request volume to detect\nsuccess rate outliers. If the number of hosts is less than this setting, outlier\ndetection via success rate statistics is not performed for any host in the\ncluster. Defaults to 5.',
    )
    successRateRequestVolume: Optional[int] = Field(
        None,
        description='The minimum number of total requests that must be collected in one interval (as\ndefined by the interval duration above) to include this host in success rate\nbased outlier detection. If the volume is lower than this setting, outlier\ndetection via success rate statistics is not performed for that host. Defaults\nto 100.',
    )
    successRateStdevFactor: Optional[int] = Field(
        None,
        description='This factor is used to determine the ejection threshold for success rate outlier\nejection. The ejection threshold is the difference between the mean success\nrate, and the product of this factor and the standard deviation of the mean\nsuccess rate: mean - (stdev * success_rate_stdev_factor). This factor is divided\nby a thousand to get a double. That is, if the desired factor is 1.9, the\nruntime value should be 1900. Defaults to 1900.',
    )


class Policy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    name: str = Field(
        ...,
        description='The name of a locality load balancer policy to be used. The value\nshould be one of the predefined ones as supported by localityLbPolicy,\nalthough at the moment only ROUND_ROBIN is supported.\n\nThis field should only be populated when the customPolicy field is not\nused.\n\nNote that specifying the same policy more than once for a backend is\nnot a valid configuration and will be rejected.\n\nThe possible values are:\n\n* \'ROUND_ROBIN\': This is a simple policy in which each healthy backend\n                is selected in round robin order.\n\n* \'LEAST_REQUEST\': An O(1) algorithm which selects two random healthy\n                  hosts and picks the host which has fewer active requests.\n\n* \'RING_HASH\': The ring/modulo hash load balancer implements consistent\n              hashing to backends. The algorithm has the property that the\n              addition/removal of a host from a set of N hosts only affects\n              1/N of the requests.\n\n* \'RANDOM\': The load balancer selects a random healthy host.\n\n* \'ORIGINAL_DESTINATION\': Backend host is selected based on the client\n                          connection metadata, i.e., connections are opened\n                          to the same address as the destination address of\n                          the incoming connection before the connection\n                          was redirected to the load balancer.\n\n* \'MAGLEV\': used as a drop in replacement for the ring hash load balancer.\n            Maglev is not as stable as ring hash but has faster table lookup\n            build times and host selection times. For more information about\n            Maglev, refer to https://ai.google/research/pubs/pub44824 Possible values: ["ROUND_ROBIN", "LEAST_REQUEST", "RING_HASH", "RANDOM", "ORIGINAL_DESTINATION", "MAGLEV"].',
    )


class SecretKeyRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    key: str = Field(..., description='Key that identifies the value to be extracted.')
    name: str = Field(..., description='Name of the Secret to extract a value from.')


class SecurityPolicyRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None,
        description='Allowed value: The `selfLink` field of a `ComputeSecurityPolicy` resource.',
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class SecurityPolicyRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ...,
        description='Allowed value: The `selfLink` field of a `ComputeSecurityPolicy` resource.',
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class SecuritySettings(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    clientTLSPolicyRef: Union[ClientTLSPolicyRef, ClientTLSPolicyRef1] = Field(
        ...,
        description='ClientTlsPolicy is a resource that specifies how a client should\nauthenticate connections to backends of a service. This resource itself\ndoes not affect configuration unless it is attached to a backend\nservice resource.',
    )
    subjectAltNames: List[str] = Field(
        ...,
        description="A list of alternate names to verify the subject identity in the certificate.\nIf specified, the client will verify that the server certificate's subject\nalt name matches one of the specified values.",
    )


class Spec(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    affinityCookieTtlSec: Optional[int] = Field(
        None,
        description='Lifetime of cookies in seconds if session_affinity is\nGENERATED_COOKIE. If set to 0, the cookie is non-persistent and lasts\nonly until the end of the browser session (or equivalent). The\nmaximum allowed value for TTL is one day.\n\nWhen the load balancing scheme is INTERNAL, this field is not used.',
    )
    backend: Optional[List[BackendItem]] = Field(
        None, description='The set of backends that serve this BackendService.'
    )
    cdnPolicy: Optional[CdnPolicy] = Field(
        None, description='Cloud CDN configuration for this BackendService.'
    )
    circuitBreakers: Optional[CircuitBreakers] = Field(
        None,
        description='Settings controlling the volume of connections to a backend service. This field\nis applicable only when the load_balancing_scheme is set to INTERNAL_SELF_MANAGED.',
    )
    compressionMode: Optional[str] = Field(
        None,
        description='Compress text responses using Brotli or gzip compression, based on the client\'s Accept-Encoding header. Possible values: ["AUTOMATIC", "DISABLED"].',
    )
    connectionDrainingTimeoutSec: Optional[int] = Field(
        None,
        description='Time for which instance will be drained (not accept new\nconnections, but still work to finish started).',
    )
    connectionTrackingPolicy: Optional[ConnectionTrackingPolicy] = Field(
        None,
        description='Connection Tracking configuration for this BackendService.\nThis is available only for Layer 4 Internal Load Balancing and\nNetwork Load Balancing.',
    )
    consistentHash: Optional[ConsistentHash] = Field(
        None,
        description='Consistent Hash-based load balancing can be used to provide soft session\naffinity based on HTTP headers, cookies or other properties. This load balancing\npolicy is applicable only for HTTP connections. The affinity to a particular\ndestination host will be lost when one or more hosts are added/removed from the\ndestination service. This field specifies parameters that control consistent\nhashing. This field only applies if the load_balancing_scheme is set to\nINTERNAL_SELF_MANAGED. This field is only applicable when locality_lb_policy is\nset to MAGLEV or RING_HASH.',
    )
    customRequestHeaders: Optional[List[str]] = Field(
        None,
        description='Headers that the HTTP/S load balancer should add to proxied\nrequests.',
    )
    customResponseHeaders: Optional[List[str]] = Field(
        None,
        description='Headers that the HTTP/S load balancer should add to proxied\nresponses.',
    )
    description: Optional[str] = Field(
        None, description='An optional description of this resource.'
    )
    edgeSecurityPolicyRef: Optional[
        Union[EdgeSecurityPolicyRef, EdgeSecurityPolicyRef1]
    ] = Field(
        None,
        description='The resource URL for the edge security policy associated with this\nbackend service.',
    )
    enableCdn: Optional[bool] = Field(
        None, description='If true, enable Cloud CDN for this BackendService.'
    )
    failoverPolicy: Optional[FailoverPolicy] = Field(
        None, description='Policy for failovers.'
    )
    healthChecks: Optional[List[Union[HealthChecks, HealthChecks1]]] = None
    iap: Optional[Union[Iap, Iap1]] = Field(
        None, description='Settings for enabling Cloud Identity Aware Proxy.'
    )
    loadBalancingScheme: Optional[str] = Field(
        None,
        description='Immutable. Indicates whether the backend service will be used with internal or\nexternal load balancing. A backend service created for one type of\nload balancing cannot be used with the other. For more information, refer to\n[Choosing a load balancer](https://cloud.google.com/load-balancing/docs/backend-service). Default value: "EXTERNAL" Possible values: ["EXTERNAL", "INTERNAL_SELF_MANAGED", "INTERNAL_MANAGED", "EXTERNAL_MANAGED"].',
    )
    localityLbPolicies: Optional[List[LocalityLbPolicy]] = Field(
        None,
        description='A list of locality load balancing policies to be used in order of\npreference. Either the policy or the customPolicy field should be set.\nOverrides any value set in the localityLbPolicy field.\n\nlocalityLbPolicies is only supported when the BackendService is referenced\nby a URL Map that is referenced by a target gRPC proxy that has the\nvalidateForProxyless field set to true.',
    )
    localityLbPolicy: Optional[str] = Field(
        None,
        description='The load balancing algorithm used within the scope of the locality.\nThe possible values are:\n\n* \'ROUND_ROBIN\': This is a simple policy in which each healthy backend\n                 is selected in round robin order.\n\n* \'LEAST_REQUEST\': An O(1) algorithm which selects two random healthy\n                   hosts and picks the host which has fewer active requests.\n\n* \'RING_HASH\': The ring/modulo hash load balancer implements consistent\n               hashing to backends. The algorithm has the property that the\n               addition/removal of a host from a set of N hosts only affects\n               1/N of the requests.\n\n* \'RANDOM\': The load balancer selects a random healthy host.\n\n* \'ORIGINAL_DESTINATION\': Backend host is selected based on the client\n                          connection metadata, i.e., connections are opened\n                          to the same address as the destination address of\n                          the incoming connection before the connection\n                          was redirected to the load balancer.\n\n* \'MAGLEV\': used as a drop in replacement for the ring hash load balancer.\n            Maglev is not as stable as ring hash but has faster table lookup\n            build times and host selection times. For more information about\n            Maglev, refer to https://ai.google/research/pubs/pub44824\n\n* \'WEIGHTED_MAGLEV\': Per-instance weighted Load Balancing via health check\n                     reported weights. If set, the Backend Service must\n                     configure a non legacy HTTP-based Health Check, and\n                     health check replies are expected to contain\n                     non-standard HTTP response header field\n                     X-Load-Balancing-Endpoint-Weight to specify the\n                     per-instance weights. If set, Load Balancing is weight\n                     based on the per-instance weights reported in the last\n                     processed health check replies, as long as every\n                     instance either reported a valid weight or had\n                     UNAVAILABLE_WEIGHT. Otherwise, Load Balancing remains\n                     equal-weight.\n\n\nThis field is applicable to either:\n\n* A regional backend service with the service_protocol set to HTTP, HTTPS, or HTTP2,\n  and loadBalancingScheme set to INTERNAL_MANAGED.\n* A global backend service with the load_balancing_scheme set to INTERNAL_SELF_MANAGED.\n* A regional backend service with loadBalancingScheme set to EXTERNAL (External Network\n  Load Balancing). Only MAGLEV and WEIGHTED_MAGLEV values are possible for External\n  Network Load Balancing. The default is MAGLEV.\n\n\nIf session_affinity is not NONE, and this field is not set to MAGLEV, WEIGHTED_MAGLEV,\nor RING_HASH, session affinity settings will not take effect.\n\nOnly ROUND_ROBIN and RING_HASH are supported when the backend service is referenced\nby a URL map that is bound to target gRPC proxy that has validate_for_proxyless\nfield set to true. Possible values: ["ROUND_ROBIN", "LEAST_REQUEST", "RING_HASH", "RANDOM", "ORIGINAL_DESTINATION", "MAGLEV", "WEIGHTED_MAGLEV"].',
    )
    location: str = Field(
        ...,
        description='Location represents the geographical location of the ComputeBackendService. Specify a region name or "global" for global resources. Reference: GCP definition of regions/zones (https://cloud.google.com/compute/docs/regions-zones/)',
    )
    logConfig: Optional[LogConfig] = Field(
        None,
        description='This field denotes the logging options for the load balancer traffic served by this backend service.\nIf logging is enabled, logs will be exported to Stackdriver.',
    )
    networkRef: Optional[Union[NetworkRef, NetworkRef1]] = Field(
        None,
        description='The network to which this backend service belongs.  This field can\nonly be specified when the load balancing scheme is set to\nINTERNAL.',
    )
    outlierDetection: Optional[OutlierDetection] = Field(
        None,
        description='Settings controlling eviction of unhealthy hosts from the load balancing pool.\nThis field is applicable only when the load_balancing_scheme is set\nto INTERNAL_SELF_MANAGED.',
    )
    portName: Optional[str] = Field(
        None,
        description='Name of backend port. The same name should appear in the instance\ngroups referenced by this service. Required when the load balancing\nscheme is EXTERNAL.',
    )
    protocol: Optional[str] = Field(
        None,
        description='The protocol this BackendService uses to communicate with backends.\nThe default is HTTP. **NOTE**: HTTP2 is only valid for beta HTTP/2 load balancer\ntypes and may result in errors if used with the GA API. **NOTE**: With protocol “UNSPECIFIED”,\nthe backend service can be used by Layer 4 Internal Load Balancing or Network Load Balancing\nwith TCP/UDP/L3_DEFAULT Forwarding Rule protocol. Possible values: ["HTTP", "HTTPS", "HTTP2", "TCP", "SSL", "GRPC", "UNSPECIFIED"].',
    )
    resourceID: Optional[str] = Field(
        None,
        description='Immutable. Optional. The name of the resource. Used for creation and acquisition. When unset, the value of `metadata.name` is used as the default.',
    )
    securityPolicy: Optional[str] = Field(
        None, description='The security policy associated with this backend service.'
    )
    securityPolicyRef: Optional[Union[SecurityPolicyRef, SecurityPolicyRef1]] = Field(
        None, description='The security policy associated with this backend service.'
    )
    securitySettings: Optional[SecuritySettings] = Field(
        None,
        description='The security settings that apply to this backend service. This field is applicable to either\na regional backend service with the service_protocol set to HTTP, HTTPS, or HTTP2, and\nload_balancing_scheme set to INTERNAL_MANAGED; or a global backend service with the\nload_balancing_scheme set to INTERNAL_SELF_MANAGED.',
    )
    sessionAffinity: Optional[str] = Field(
        None,
        description='Type of session affinity to use. The default is NONE. Session affinity is\nnot applicable if the protocol is UDP. Possible values: ["NONE", "CLIENT_IP", "CLIENT_IP_PORT_PROTO", "CLIENT_IP_PROTO", "GENERATED_COOKIE", "HEADER_FIELD", "HTTP_COOKIE"].',
    )
    subsetting: Optional[Subsetting] = Field(
        None,
        description='Subsetting configuration for this BackendService. Currently this is applicable only for Internal TCP/UDP load balancing and Internal HTTP(S) load balancing.',
    )
    timeoutSec: Optional[int] = Field(
        None,
        description='How many seconds to wait for the backend before considering it a\nfailed request. Default is 30 seconds. Valid range is [1, 86400].',
    )


class Status(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    conditions: Optional[List[Condition]] = Field(
        None,
        description="Conditions represent the latest available observation of the resource's current state.",
    )
    creationTimestamp: Optional[str] = Field(
        None, description='Creation timestamp in RFC3339 text format.'
    )
    fingerprint: Optional[str] = Field(
        None,
        description='Fingerprint of this resource. A hash of the contents stored in this\nobject. This field is used in optimistic locking.',
    )
    generatedId: Optional[int] = Field(
        None,
        description='The unique identifier for the resource. This identifier is defined by the server.',
    )
    observedGeneration: Optional[int] = Field(
        None,
        description='ObservedGeneration is the generation of the resource that was most recently observed by the Config Connector controller. If this is equal to metadata.generation, then that means that the current reported status reflects the most recent desired state of the resource.',
    )
    selfLink: Optional[str] = None


class Subsetting(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    policy: str = Field(
        ...,
        description='The algorithm used for subsetting. Possible values: ["CONSISTENT_HASH_SUBSETTING"].',
    )


class Ttl(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    nanos: Optional[int] = Field(
        None,
        description="Span of time that's a fraction of a second at nanosecond\nresolution. Durations less than one second are represented\nwith a 0 seconds field and a positive nanos field. Must\nbe from 0 to 999,999,999 inclusive.",
    )
    seconds: int = Field(
        ...,
        description='Span of time at a resolution of a second.\nMust be from 0 to 315,576,000,000 inclusive.',
    )


class ValueFrom(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    secretKeyRef: Optional[SecretKeyRef] = Field(
        None,
        description="Reference to a value with the given key in the given Secret in the resource's namespace.",
    )


class ValueFrom1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    secretKeyRef: Optional[SecretKeyRef] = Field(
        None,
        description="Reference to a value with the given key in the given Secret in the resource's namespace.",
    )


class ValueFrom2(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    secretKeyRef: Optional[SecretKeyRef] = Field(
        None,
        description="Reference to a value with the given key in the given Secret in the resource's namespace.",
    )


class ValueFrom3(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    secretKeyRef: Optional[SecretKeyRef] = Field(
        None,
        description="Reference to a value with the given key in the given Secret in the resource's namespace.",
    )
