# generated by datamodel-codegen:
#   filename:  computeregionautoscaler_v1alpha1.json
#   timestamp: 2024-03-15T04:51:48+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field


class AutoscalingPolicy(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    cooldownPeriod: Optional[int] = Field(
        None,
        description='The number of seconds that the autoscaler should wait before it\nstarts collecting information from a new instance. This prevents\nthe autoscaler from collecting information when the instance is\ninitializing, during which the collected usage would not be\nreliable. The default time autoscaler waits is 60 seconds.\n\nVirtual machine initialization times might vary because of\nnumerous factors. We recommend that you test how long an\ninstance may take to initialize. To do this, create an instance\nand time the startup process.',
    )
    cpuUtilization: Optional[CpuUtilization] = Field(
        None,
        description='Defines the CPU utilization policy that allows the autoscaler to\nscale based on the average CPU utilization of a managed instance\ngroup.',
    )
    loadBalancingUtilization: Optional[LoadBalancingUtilization] = Field(
        None,
        description='Configuration parameters of autoscaling based on a load balancer.',
    )
    maxReplicas: int = Field(
        ...,
        description='The maximum number of instances that the autoscaler can scale up\nto. This is required when creating or updating an autoscaler. The\nmaximum number of replicas should not be lower than minimal number\nof replicas.',
    )
    metric: Optional[List[MetricItem]] = Field(
        None,
        description='Configuration parameters of autoscaling based on a custom metric.',
    )
    minReplicas: int = Field(
        ...,
        description='The minimum number of replicas that the autoscaler can scale down\nto. This cannot be less than 0. If not provided, autoscaler will\nchoose a default value depending on maximum number of instances\nallowed.',
    )
    mode: Optional[str] = Field(
        None, description='Defines operating mode for this policy.'
    )
    scaleDownControl: Optional[ScaleDownControl] = Field(
        None,
        description='Defines scale down controls to reduce the risk of response latency\nand outages due to abrupt scale-in events.',
    )
    scaleInControl: Optional[ScaleInControl] = Field(
        None,
        description='Defines scale in controls to reduce the risk of response latency\nand outages due to abrupt scale-in events.',
    )
    scalingSchedules: Optional[List[ScalingSchedule]] = Field(
        None,
        description='Scaling schedules defined for an autoscaler. Multiple schedules can be set on an autoscaler and they can overlap.',
    )


class Condition(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    lastTransitionTime: Optional[str] = Field(
        None,
        description='Last time the condition transitioned from one status to another.',
    )
    message: Optional[str] = Field(
        None,
        description='Human-readable message indicating details about last transition.',
    )
    reason: Optional[str] = Field(
        None,
        description="Unique, one-word, CamelCase reason for the condition's last transition.",
    )
    status: Optional[str] = Field(
        None,
        description='Status is the status of the condition. Can be True, False, Unknown.',
    )
    type: Optional[str] = Field(None, description='Type is the type of the condition.')


class CpuUtilization(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    predictiveMethod: Optional[str] = Field(
        None,
        description='Indicates whether predictive autoscaling based on CPU metric is enabled. Valid values are:\n\n- NONE (default). No predictive method is used. The autoscaler scales the group to meet current demand based on real-time metrics.\n\n- OPTIMIZE_AVAILABILITY. Predictive autoscaling improves availability by monitoring daily and weekly load patterns and scaling out ahead of anticipated demand.',
    )
    target: float = Field(
        ...,
        description='The target CPU utilization that the autoscaler should maintain.\nMust be a float value in the range (0, 1]. If not specified, the\ndefault is 0.6.\n\nIf the CPU level is below the target utilization, the autoscaler\nscales down the number of instances until it reaches the minimum\nnumber of instances you specified or until the average CPU of\nyour instances reaches the target utilization.\n\nIf the average CPU is above the target utilization, the autoscaler\nscales up until it reaches the maximum number of instances you\nspecified or until the average utilization reaches the target\nutilization.',
    )


class LoadBalancingUtilization(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    target: float = Field(
        ...,
        description='Fraction of backend capacity utilization (set in HTTP(s) load\nbalancing configuration) that autoscaler should maintain. Must\nbe a positive float value. If not defined, the default is 0.8.',
    )


class MaxScaledDownReplicas(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    fixed: Optional[int] = Field(
        None,
        description='Specifies a fixed number of VM instances. This must be a positive\ninteger.',
    )
    percent: Optional[int] = Field(
        None,
        description='Specifies a percentage of instances between 0 to 100%, inclusive.\nFor example, specify 80 for 80%.',
    )


class MaxScaledInReplicas(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    fixed: Optional[int] = Field(
        None,
        description='Specifies a fixed number of VM instances. This must be a positive\ninteger.',
    )
    percent: Optional[int] = Field(
        None,
        description='Specifies a percentage of instances between 0 to 100%, inclusive.\nFor example, specify 80 for 80%.',
    )


class MetricItem(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    filter: Optional[str] = Field(
        None,
        description='A filter string to be used as the filter string for\na Stackdriver Monitoring TimeSeries.list API call.\nThis filter is used to select a specific TimeSeries for\nthe purpose of autoscaling and to determine whether the metric\nis exporting per-instance or per-group data.\n\nYou can only use the AND operator for joining selectors.\nYou can only use direct equality comparison operator (=) without\nany functions for each selector.\nYou can specify the metric in both the filter string and in the\nmetric field. However, if specified in both places, the metric must\nbe identical.\n\nThe monitored resource type determines what kind of values are\nexpected for the metric. If it is a gce_instance, the autoscaler\nexpects the metric to include a separate TimeSeries for each\ninstance in a group. In such a case, you cannot filter on resource\nlabels.\n\nIf the resource type is any other value, the autoscaler expects\nthis metric to contain values that apply to the entire autoscaled\ninstance group and resource label filtering can be performed to\npoint autoscaler at the correct TimeSeries to scale upon.\nThis is called a per-group metric for the purpose of autoscaling.\n\nIf not specified, the type defaults to gce_instance.\n\nYou should provide a filter that is selective enough to pick just\none TimeSeries for the autoscaled group or for each of the instances\n(if you are using gce_instance resource type). If multiple\nTimeSeries are returned upon the query execution, the autoscaler\nwill sum their respective values to obtain its scaling value.',
    )
    name: str = Field(
        ...,
        description='The identifier (type) of the Stackdriver Monitoring metric.\nThe metric cannot have negative values.\n\nThe metric must have a value type of INT64 or DOUBLE.',
    )
    singleInstanceAssignment: Optional[float] = Field(
        None,
        description="If scaling is based on a per-group metric value that represents the\ntotal amount of work to be done or resource usage, set this value to\nan amount assigned for a single instance of the scaled group.\nThe autoscaler will keep the number of instances proportional to the\nvalue of this metric, the metric itself should not change value due\nto group resizing.\n\nFor example, a good metric to use with the target is\n'pubsub.googleapis.com/subscription/num_undelivered_messages'\nor a custom metric exporting the total number of requests coming to\nyour instances.\n\nA bad example would be a metric exporting an average or median\nlatency, since this value can't include a chunk assignable to a\nsingle instance, it could be better used with utilization_target\ninstead.",
    )
    target: Optional[float] = Field(
        None,
        description='The target value of the metric that autoscaler should\nmaintain. This must be a positive value. A utilization\nmetric scales number of virtual machines handling requests\nto increase or decrease proportionally to the metric.\n\nFor example, a good metric to use as a utilizationTarget is\nwww.googleapis.com/compute/instance/network/received_bytes_count.\nThe autoscaler will work to keep this value constant for each\nof the instances.',
    )
    type: Optional[str] = Field(
        None,
        description='Defines how target utilization value is expressed for a\nStackdriver Monitoring metric. Possible values: ["GAUGE", "DELTA_PER_SECOND", "DELTA_PER_MINUTE"].',
    )


class Model(BaseModel):
    apiVersion: Optional[str] = Field(
        None,
        description='apiVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources',
    )
    kind: Optional[str] = Field(
        None,
        description='kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds',
    )
    metadata: Optional[Dict[str, Any]] = None
    spec: Spec
    status: Optional[Status] = None


class ProjectRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None, description='Allowed value: The `name` field of a `Project` resource.'
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class ProjectRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ..., description='Allowed value: The `name` field of a `Project` resource.'
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class ScaleDownControl(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    maxScaledDownReplicas: Optional[MaxScaledDownReplicas] = Field(
        None, description='A nested object resource.'
    )
    timeWindowSec: Optional[int] = Field(
        None,
        description='How long back autoscaling should look when computing recommendations\nto include directives regarding slower scale down, as described above.',
    )


class ScaleInControl(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    maxScaledInReplicas: Optional[MaxScaledInReplicas] = Field(
        None, description='A nested object resource.'
    )
    timeWindowSec: Optional[int] = Field(
        None,
        description='How long back autoscaling should look when computing recommendations\nto include directives regarding slower scale down, as described above.',
    )


class ScalingSchedule(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    description: Optional[str] = Field(
        None, description='A description of a scaling schedule.'
    )
    disabled: Optional[bool] = Field(
        None,
        description='A boolean value that specifies if a scaling schedule can influence autoscaler recommendations. If set to true, then a scaling schedule has no effect.',
    )
    durationSec: int = Field(
        ...,
        description='The duration of time intervals (in seconds) for which this scaling schedule will be running. The minimum allowed value is 300.',
    )
    minRequiredReplicas: int = Field(
        ...,
        description='Minimum number of VM instances that autoscaler will recommend in time intervals starting according to schedule.',
    )
    name: str
    schedule: str = Field(
        ...,
        description='The start timestamps of time intervals when this scaling schedule should provide a scaling signal. This field uses the extended cron format (with an optional year field).',
    )
    timeZone: Optional[str] = Field(
        None,
        description='The time zone to be used when interpreting the schedule. The value of this field must be a time zone name from the tz database: http://en.wikipedia.org/wiki/Tz_database.',
    )


class Spec(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    autoscalingPolicy: AutoscalingPolicy = Field(
        ...,
        description='The configuration parameters for the autoscaling algorithm. You can\ndefine one or more of the policies for an autoscaler: cpuUtilization,\ncustomMetricUtilizations, and loadBalancingUtilization.\n\nIf none of these are specified, the default will be to autoscale based\non cpuUtilization to 0.6 or 60%.',
    )
    description: Optional[str] = Field(
        None, description='An optional description of this resource.'
    )
    projectRef: Union[ProjectRef, ProjectRef1] = Field(
        ..., description='The project that this resource belongs to.'
    )
    region: str = Field(
        ...,
        description='Immutable. URL of the region where the instance group resides.',
    )
    resourceID: Optional[str] = Field(
        None,
        description='Immutable. Optional. The name of the resource. Used for creation and acquisition. When unset, the value of `metadata.name` is used as the default.',
    )
    target: str = Field(
        ...,
        description='URL of the managed instance group that this autoscaler will scale.',
    )


class Status(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    conditions: Optional[List[Condition]] = Field(
        None,
        description="Conditions represent the latest available observation of the resource's current state.",
    )
    creationTimestamp: Optional[str] = Field(
        None, description='Creation timestamp in RFC3339 text format.'
    )
    observedGeneration: Optional[int] = Field(
        None,
        description='ObservedGeneration is the generation of the resource that was most recently observed by the Config Connector controller. If this is equal to metadata.generation, then that means that the current reported status reflects the most recent desired state of the resource.',
    )
    selfLink: Optional[str] = None
