# generated by datamodel-codegen:
#   filename:  vertexaiendpoint_v1alpha1.json
#   timestamp: 2024-03-15T04:53:38+00:00

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field


class AutomaticResource(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    maxReplicaCount: Optional[int] = Field(
        None,
        description='The maximum number of replicas this DeployedModel may be deployed on when the traffic against it increases. If the requested value is too large, the deployment will error, but if deployment succeeds then the ability to scale the model to that many replicas is guaranteed (barring service outages). If traffic against the DeployedModel increases beyond what its replicas at maximum may handle, a portion of the traffic will be dropped. If this value is not provided, a no upper bound for scaling under heavy traffic will be assume, though Vertex AI may be unable to scale beyond certain replica number.',
    )
    minReplicaCount: Optional[int] = Field(
        None,
        description='The minimum number of replicas this DeployedModel will be always deployed on. If traffic against it increases, it may dynamically be deployed onto more replicas up to max_replica_count, and as traffic decreases, some of these extra replicas may be freed. If the requested value is too large, the deployment will error.',
    )


class AutoscalingMetricSpec(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    metricName: Optional[str] = Field(
        None,
        description="The resource metric name. Supported metrics: * For Online Prediction: * 'aiplatform.googleapis.com/prediction/online/accelerator/duty_cycle' * 'aiplatform.googleapis.com/prediction/online/cpu/utilization'.",
    )
    target: Optional[int] = Field(
        None,
        description='The target resource utilization in percentage (1% - 100%) for the given metric; once the real usage deviates from the target by a certain percentage, the machine replicas change. The default value is 60 (representing 60%) if not provided.',
    )


class Condition(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    lastTransitionTime: Optional[str] = Field(
        None,
        description='Last time the condition transitioned from one status to another.',
    )
    message: Optional[str] = Field(
        None,
        description='Human-readable message indicating details about last transition.',
    )
    reason: Optional[str] = Field(
        None,
        description="Unique, one-word, CamelCase reason for the condition's last transition.",
    )
    status: Optional[str] = Field(
        None,
        description='Status is the status of the condition. Can be True, False, Unknown.',
    )
    type: Optional[str] = Field(None, description='Type is the type of the condition.')


class DedicatedResource(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    autoscalingMetricSpecs: Optional[List[AutoscalingMetricSpec]] = Field(
        None,
        description="The metric specifications that overrides a resource utilization metric (CPU utilization, accelerator's duty cycle, and so on) target value (default to 60 if not set). At most one entry is allowed per metric. If machine_spec.accelerator_count is above 0, the autoscaling will be based on both CPU utilization and accelerator's duty cycle metrics and scale up when either metrics exceeds its target value while scale down if both metrics are under their target value. The default target value is 60 for both metrics. If machine_spec.accelerator_count is 0, the autoscaling will be based on CPU utilization metric only with default target value 60 if not explicitly set. For example, in the case of Online Prediction, if you want to override target CPU utilization to 80, you should set autoscaling_metric_specs.metric_name to 'aiplatform.googleapis.com/prediction/online/cpu/utilization' and autoscaling_metric_specs.target to '80'.",
    )
    machineSpec: Optional[List[MachineSpecItem]] = Field(
        None,
        description='The specification of a single machine used by the prediction.',
    )
    maxReplicaCount: Optional[int] = Field(
        None,
        description='The maximum number of replicas this DeployedModel may be deployed on when the traffic against it increases. If the requested value is too large, the deployment will error, but if deployment succeeds then the ability to scale the model to that many replicas is guaranteed (barring service outages). If traffic against the DeployedModel increases beyond what its replicas at maximum may handle, a portion of the traffic will be dropped. If this value is not provided, will use min_replica_count as the default value. The value of this field impacts the charge against Vertex CPU and GPU quotas. Specifically, you will be charged for max_replica_count * number of cores in the selected machine type) and (max_replica_count * number of GPUs per replica in the selected machine type).',
    )
    minReplicaCount: Optional[int] = Field(
        None,
        description='The minimum number of machine replicas this DeployedModel will be always deployed on. This value must be greater than or equal to 1. If traffic against the DeployedModel increases, it may dynamically be deployed onto more replicas, and as traffic decreases, some of these extra replicas may be freed.',
    )


class DeployedModel(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    automaticResources: Optional[List[AutomaticResource]] = Field(
        None,
        description='A description of resources that to large degree are decided by Vertex AI, and require only a modest additional configuration.',
    )
    createTime: Optional[str] = Field(
        None, description='Output only. Timestamp when the DeployedModel was created.'
    )
    dedicatedResources: Optional[List[DedicatedResource]] = Field(
        None,
        description='A description of resources that are dedicated to the DeployedModel, and that need a higher degree of manual configuration.',
    )
    displayName: Optional[str] = Field(
        None,
        description="The display name of the DeployedModel. If not provided upon creation, the Model's display_name is used.",
    )
    enableAccessLogging: Optional[bool] = Field(
        None,
        description='These logs are like standard server access logs, containing information like timestamp and latency for each prediction request. Note that Stackdriver logs may incur a cost, especially if your project receives prediction requests at a high queries per second rate (QPS). Estimate your costs before enabling this option.',
    )
    enableContainerLogging: Optional[bool] = Field(
        None,
        description="If true, the container of the DeployedModel instances will send 'stderr' and 'stdout' streams to Stackdriver Logging. Only supported for custom-trained Models and AutoML Tabular Models.",
    )
    id: Optional[str] = Field(
        None,
        description='The ID of the DeployedModel. If not provided upon deployment, Vertex AI will generate a value for this ID. This value should be 1-10 characters, and valid characters are /[0-9]/.',
    )
    model: Optional[str] = Field(
        None,
        description="The name of the Model that this is the deployment of. Note that the Model may be in a different location than the DeployedModel's Endpoint.",
    )
    modelVersionId: Optional[str] = Field(
        None, description='Output only. The version ID of the model that is deployed.'
    )
    privateEndpoints: Optional[List[PrivateEndpoint]] = Field(
        None,
        description='Output only. Provide paths for users to send predict/explain/health requests directly to the deployed model services running on Cloud via private services access. This field is populated if network is configured.',
    )
    serviceAccount: Optional[str] = Field(
        None,
        description="The service account that the DeployedModel's container runs as. Specify the email address of the service account. If this service account is not specified, the container runs as a service account that doesn't have access to the resource project. Users deploying the Model must have the 'iam.serviceAccounts.actAs' permission on this service account.",
    )
    sharedResources: Optional[str] = Field(
        None,
        description='The resource name of the shared DeploymentResourcePool to deploy on. Format: projects/{project}/locations/{location}/deploymentResourcePools/{deployment_resource_pool}.',
    )


class EncryptionSpec(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    kmsKeyName: str = Field(
        ...,
        description="Immutable. Required. The Cloud KMS resource identifier of the customer managed encryption key used to protect a resource. Has the form: 'projects/my-project/locations/my-region/keyRings/my-kr/cryptoKeys/my-key'. The key needs to be in the same region as where the compute resource is created.",
    )


class MachineSpecItem(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    acceleratorCount: Optional[int] = Field(
        None, description='The number of accelerators to attach to the machine.'
    )
    acceleratorType: Optional[str] = Field(
        None,
        description='The type of accelerator(s) that may be attached to the machine as per accelerator_count. See possible values [here](https://cloud.google.com/vertex-ai/docs/reference/rest/v1/MachineSpec#AcceleratorType).',
    )
    machineType: Optional[str] = Field(
        None,
        description="The type of the machine. See the [list of machine types supported for prediction](https://cloud.google.com/vertex-ai/docs/predictions/configure-compute#machine-types) See the [list of machine types supported for custom training](https://cloud.google.com/vertex-ai/docs/training/configure-compute#machine-types). For DeployedModel this field is optional, and the default value is 'n1-standard-2'. For BatchPredictionJob or as part of WorkerPoolSpec this field is required. TODO(rsurowka): Try to better unify the required vs optional.",
    )


class Model(BaseModel):
    apiVersion: Optional[str] = Field(
        None,
        description='apiVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#resources',
    )
    kind: Optional[str] = Field(
        None,
        description='kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/api-conventions.md#types-kinds',
    )
    metadata: Optional[Dict[str, Any]] = None
    spec: Spec
    status: Optional[Status] = None


class PrivateEndpoint(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    explainHttpUri: Optional[str] = Field(
        None, description='Output only. Http(s) path to send explain requests.'
    )
    healthHttpUri: Optional[str] = Field(
        None, description='Output only. Http(s) path to send health check requests.'
    )
    predictHttpUri: Optional[str] = Field(
        None, description='Output only. Http(s) path to send prediction requests.'
    )
    serviceAttachment: Optional[str] = Field(
        None,
        description='Output only. The name of the service attachment resource. Populated if private service connect is enabled.',
    )


class ProjectRef(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: Optional[str] = Field(
        None, description='Allowed value: The `name` field of a `Project` resource.'
    )
    name: str = Field(
        ...,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class ProjectRef1(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    external: str = Field(
        ..., description='Allowed value: The `name` field of a `Project` resource.'
    )
    name: Optional[str] = Field(
        None,
        description='Name of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/names/#names',
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace of the referent. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/namespaces/',
    )


class Spec(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    description: Optional[str] = Field(
        None, description='The description of the Endpoint.'
    )
    displayName: str = Field(
        ...,
        description='Required. The display name of the Endpoint. The name can be up to 128 characters long and can consist of any UTF-8 characters.',
    )
    encryptionSpec: Optional[EncryptionSpec] = Field(
        None,
        description='Immutable. Customer-managed encryption key spec for an Endpoint. If set, this Endpoint and all sub-resources of this Endpoint will be secured by this key.',
    )
    location: str = Field(..., description='Immutable. The location for the resource.')
    network: Optional[str] = Field(
        None,
        description="Immutable. The full name of the Google Compute Engine [network](https://cloud.google.com//compute/docs/networks-and-firewalls#networks) to which the Endpoint should be peered. Private services access must already be configured for the network. If left unspecified, the Endpoint is not peered with any network. Only one of the fields, network or enable_private_service_connect, can be set. [Format](https://cloud.google.com/compute/docs/reference/rest/v1/networks/insert): 'projects/{project}/global/networks/{network}'. Where '{project}' is a project number, as in '12345', and '{network}' is network name.",
    )
    projectRef: Union[ProjectRef, ProjectRef1] = Field(
        ..., description='The project that this resource belongs to.'
    )
    region: Optional[str] = Field(
        None, description='Immutable. The region for the resource.'
    )
    resourceID: Optional[str] = Field(
        None,
        description='Immutable. Optional. The name of the resource. Used for creation and acquisition. When unset, the value of `metadata.name` is used as the default.',
    )


class Status(BaseModel):
    model_config = ConfigDict(
        extra='forbid',
    )
    conditions: Optional[List[Condition]] = Field(
        None,
        description="Conditions represent the latest available observation of the resource's current state.",
    )
    createTime: Optional[str] = Field(
        None, description='Output only. Timestamp when this Endpoint was created.'
    )
    deployedModels: Optional[List[DeployedModel]] = Field(
        None,
        description='Output only. The models deployed in this Endpoint. To add or remove DeployedModels use EndpointService.DeployModel and EndpointService.UndeployModel respectively. Models can also be deployed and undeployed using the [Cloud Console](https://console.cloud.google.com/vertex-ai/).',
    )
    etag: Optional[str] = Field(
        None,
        description='Used to perform consistent read-modify-write updates. If not set, a blind "overwrite" update happens.',
    )
    modelDeploymentMonitoringJob: Optional[str] = Field(
        None,
        description="Output only. Resource name of the Model Monitoring job associated with this Endpoint if monitoring is enabled by CreateModelDeploymentMonitoringJob. Format: 'projects/{project}/locations/{location}/modelDeploymentMonitoringJobs/{model_deployment_monitoring_job}'.",
    )
    observedGeneration: Optional[int] = Field(
        None,
        description='ObservedGeneration is the generation of the resource that was most recently observed by the Config Connector controller. If this is equal to metadata.generation, then that means that the current reported status reflects the most recent desired state of the resource.',
    )
    updateTime: Optional[str] = Field(
        None, description='Output only. Timestamp when this Endpoint was last updated.'
    )
